package edu.kit.ipd.descartes.adaptation.evaluation.weightingfunction;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.junit.Before;
import org.junit.Test;

import edu.kit.ipd.descartes.adaptation.evaluation.weightingfunction.WeightedSumWeightingFunctionHelper;
import edu.kit.ipd.descartes.adaptation.model.perfdatarepo.PerfDataRepoConverter;
import edu.kit.ipd.descartes.adaptation.model.perfdatarepo.PerformanceDataRepositoryModelLoader;
import edu.kit.ipd.descartes.mm.adaptation.AdaptationFactory;
import edu.kit.ipd.descartes.mm.adaptation.WeightedMetric;
import edu.kit.ipd.descartes.mm.adaptation.WeightingFunction;
import edu.kit.ipd.descartes.perfdatarepo.Impact;
import edu.kit.ipd.descartes.perfdatarepo.MetricType;
import edu.kit.ipd.descartes.perfdatarepo.MetricValue;
import edu.kit.ipd.descartes.perfdatarepo.PerfdatarepoFactory;
import edu.kit.ipd.descartes.perfdatarepo.PerformanceDataRepository;
import edu.kit.ipd.descartes.perfdatarepo.Result;
import edu.kit.ipd.descartes.ql.models.mapping.domain.DecimalResult;
import edu.kit.ipd.descartes.ql.models.mapping.domain.DomainFactory;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Resource;
import edu.kit.ipd.descartes.ql.models.mapping.mapping.EntityMapping;
import edu.kit.ipd.descartes.ql.models.mapping.mapping.MappingFactory;

public class WeightingFunctionHelperTest {

    private static final String PERF_DATA_REPO_XMI_FILE = "./test_models/PerformanceDataRepositoryDummy.xmi";
    private static final double UTIL_WEIGHT = 2.0d;

    private PerformanceDataRepositoryModelLoader perfDataModelManager = PerformanceDataRepositoryModelLoader.getInstance();
    private PerformanceDataRepository perfDataRepo = null;
    private WeightingFunction function = null;

    private enum METRIC_TYPES {
        UTILIZATION, RESPONSE_TIME;
    }

    private MetricType getTypeHelper(METRIC_TYPES mType) {
        EList<MetricType> types = perfDataRepo.getMetricTypes();
        switch (mType) {
        case UTILIZATION:
            return types.get(0);
        case RESPONSE_TIME:
            return types.get(1);
        default:
            return null;
        }
    }

    @Before
    public void setUp() throws Exception {
        perfDataRepo = (PerformanceDataRepository) perfDataModelManager.load(URI.createFileURI(PERF_DATA_REPO_XMI_FILE));
        MetricType metricType = getTypeHelper(METRIC_TYPES.UTILIZATION);

        function = AdaptationFactory.eINSTANCE.createWeightingFunction();
        WeightedMetric wUtil = AdaptationFactory.eINSTANCE.createWeightedMetric();
        wUtil.setId("1");
        wUtil.setName("UtilizationMetricWeight");
        wUtil.setWeight(UTIL_WEIGHT);
        wUtil.setMetricType(metricType);
        function.getWeightedMetrics().add(wUtil);
    }

    @Test
    public void testGetWeightForMetricType() {
        double weight = 0.0;
        MetricType metricType = getTypeHelper(METRIC_TYPES.RESPONSE_TIME);
        weight = WeightedSumWeightingFunctionHelper.getWeightForMetricType(function, metricType);
        assertEquals(Double.NaN, weight, 0.0);

        metricType = getTypeHelper(METRIC_TYPES.UTILIZATION);
        weight = WeightedSumWeightingFunctionHelper.getWeightForMetricType(function, metricType);
        assertEquals(UTIL_WEIGHT, weight, 0.0);
    }

    @Test
    public void testGetValueForMetricType() {
        double testValue = 0;
        MetricType metricType = getTypeHelper(METRIC_TYPES.UTILIZATION);

        Result emptyResult = PerfdatarepoFactory.eINSTANCE.createResult();
        testValue = WeightedSumWeightingFunctionHelper.getValueForMetricType(metricType, emptyResult);
        assertEquals(Double.NaN, testValue, 0);

        Impact impact = perfDataRepo.getImpactHistory().get(0);
        Result before = impact.getBefore();
        testValue = WeightedSumWeightingFunctionHelper.getValueForMetricType(metricType, before);
        assertEquals(0.4, testValue, 0);
    }
    
    @Test
    public void testSameMetricType() {
        Resource r1 = DomainFactory.eINSTANCE.createResource();
        Resource r2 = DomainFactory.eINSTANCE.createResource();
        
        DecimalResult p1 = DomainFactory.eINSTANCE.createDecimalResult();
        DecimalResult p2 = DomainFactory.eINSTANCE.createDecimalResult();
        DecimalResult p3 = DomainFactory.eINSTANCE.createDecimalResult();

        EntityMapping m = MappingFactory.eINSTANCE.createEntityMapping();
        
        p1.setAccuracy(new BigDecimal(0.0));
        p1.setMetricName("ResponseTimeMetricNameTest");
        p1.setValid(Boolean.TRUE);
        p1.setValue(new BigDecimal(100.0));
        
        p2.setAccuracy(new BigDecimal(0.0));
        p2.setMetricName("ResponseTimeMetricNameTest");
        p2.setValid(Boolean.TRUE);
        p2.setValue(new BigDecimal(110.0));
        
        p3.setAccuracy(new BigDecimal(0.0));
        p3.setMetricName("ResponseTimeMetricNameTest");
        p3.setValid(Boolean.TRUE);
        p3.setValue(new BigDecimal(100.0));
        
        r1.setAlias("Resource1");
        r1.setIdentifier("Resource1-ID");
        r1.getProbes().add(p1);
        r1.getProbes().add(p2);
        
        r2.setAlias("Resource2");
        r2.setIdentifier("Resource2-ID");
        r2.getProbes().add(p3);
        
        m.getResources().add(r1);
        m.getResources().add(r2);
        
        Collection<MetricValue> result = PerfDataRepoConverter.convertMapping(m);
        Iterator<MetricValue> i = result.iterator();
        MetricValue v1 = i.next();
        MetricValue v2 = i.next();
        assertEquals(v1.getMetricType(), v2.getMetricType());
        MetricValue v3 = i.next();
        assertTrue(WeightedSumWeightingFunctionHelper.sameMetricType(v2, v3.getMetricType()));
    }

}
